/**
 ** Parchis4A.java
 **
 ** Copyright (C) 1992, 1999, 2007, 2011 Mariano Alvarez Fernandez
 ** [e-mail: malfer at telefonica.net]
 **
 ** This file is part of the Parchis4A program.
 **
 ** Parchis4A is free software: you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation, either version 3 of the License, or
 ** (at your option) any later version.
 **
 ** Parchis4A is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with Parchis4A.  If not, see <http://www.gnu.org/licenses/>.
 **
 **/

package com.fgrim.parchis4a;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.StreamCorruptedException;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.res.Resources;
import android.graphics.PorterDuff;
import android.graphics.drawable.AnimationDrawable;
import android.graphics.drawable.Drawable;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.preference.PreferenceManager;
import android.util.Log;
import android.view.Gravity;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.animation.Animation;
import android.view.animation.AnimationUtils;
import android.widget.Button;
import android.widget.RadioGroup;
import android.widget.TextView;
import android.widget.Toast;

public class Parchis4A extends Activity {

//	private static final String TAG = "Parchis4A";
    private static final String SAVE_FILENAME = "parchis4a_save.bin";
	// This is incremented when the save system changes.
	private static final String SAVE_VERSION = "parchis4a_save_3";
	
    public static final int DIALOG_SELECTPLAYERS_ID = 0;
    public static final int DIALOG_ABOUT_ID = 1;
    public static final int DIALOG_HELP_ID = 2;
    public static final int DIALOG_EXIT_ID = 3;
    public static final int DIALOG_WELCOME_ID = 4;
    public static final int DIALOG_GAMESPEED_ID = 5;
    public static final int DIALOG_STATS_ID = 6;
    public static final int DIALOG_NEW_ID = 7;

	private static final int colorj[] = {0xffffff55, 0xff5555ff, 0xffff5555, 0xff00aa00};

	private static String ICICLE_KEY = "parchis4a-state";
	
	private Resources resG;

	private int autointv = 10;
	private int autointvItem = 0;
	
	private int tipo_jugador[];
	private EstadoPartida mEP;

	private ParchisView mParchisView;
	private TextView mTVOnPause;
//	private TextView mTVEstadisticas;
	private Button mTirador;
	private Button mAuxiliar;
	private Button mBPausa;
//	private Button mBDadoN[] = new Button[6];
	private Tablero mTablero;
	private RadioGroup mRG_tipo_jugador[];
	
	private int blinkPawn = 0;
	private Carrera mCarrera;
	private boolean enCarrera = false;
	private MarchaPodium mMarchaPodium;
	private boolean enMarchaPodium = false;
	
	private int enPausa = 0;
	private boolean enPausaManual = false;
	private boolean enPausaHandle = false;
	private boolean enPausaDado = false;
	private boolean arrancando = true;
	
	private AnimationDrawable dadoAnimation;
	private boolean dadoCurrando = false;
	
	private int ultVerExe = 0;
	
    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        
        resG = getResources();
        setContentView(R.layout.main);
        tipo_jugador = new int[4];
        mRG_tipo_jugador = new RadioGroup[4];
        tipo_jugador[0] = EstadoPartida.PERSONA;
        tipo_jugador[1] = EstadoPartida.MAQUINA;
        tipo_jugador[2] = EstadoPartida.MAQUINA;
        tipo_jugador[3] = EstadoPartida.MAQUINA;
        mParchisView = (ParchisView) findViewById(R.id.parchis);
        mTirador = (Button) findViewById(R.id.btirador);
        mAuxiliar = (Button) findViewById(R.id.bauxiliar);
        mBPausa = (Button) findViewById(R.id.bpausa);
        mTVOnPause = (TextView) findViewById(R.id.parchis_onpause);
//        mTVEstadisticas = (TextView) findViewById(R.id.parchis_estadisticas);
//        mBDadoN[0] = (Button) findViewById(R.id.bdado1);
//        mBDadoN[1] = (Button) findViewById(R.id.bdado2);
//        mBDadoN[2] = (Button) findViewById(R.id.bdado3);
//        mBDadoN[3] = (Button) findViewById(R.id.bdado4);
//        mBDadoN[4] = (Button) findViewById(R.id.bdado5);
//        mBDadoN[5] = (Button) findViewById(R.id.bdado6);
        getPreferences();

        if (!restoreState(savedInstanceState)) {
//        	Bundle bSave = loadSavedGame();
//        	if (bSave == null || !restoreState(bSave)) {
        	    setNewGame();
                showDialog(DIALOG_WELCOME_ID);
//        	}
        }
    }
    
    @Override
    public void onSaveInstanceState(Bundle outState) {
    	super.onSaveInstanceState(outState);
        //Store the game state
    	Bundle estado = mEP.saveState();
        estado.putInt("n0aitv", autointv);
        estado.putInt("n0aitvi", autointvItem);
        estado.putIntArray("n0tj", tipo_jugador);
        estado.putBoolean("n0epm", enPausaManual);
        outState.putBundle(ICICLE_KEY, estado);
//        Log.d(TAG, "onSveState "+enPausa);
   }
    
    public void getPreferences() {
        SharedPreferences prefs =
        	PreferenceManager.getDefaultSharedPreferences(getBaseContext());
        String boardImage = prefs.getString("BoardImage", "1");
        mParchisView.setBoardImage(boardImage);
        mParchisView.showMunecos = prefs.getBoolean("ShowMunecos", true);
        mParchisView.showDestino = prefs.getBoolean("ShowDestino", false);
        mParchisView.showPlPawns = prefs.getBoolean("ShowPlPawns", false);
        autointvItem = prefs.getInt("AutointvItem", 0);
    	autointv = resG.getIntArray(R.array.dlg_gamespeed_value)[autointvItem];
    	ultVerExe = prefs.getInt("UltVerExe", 0);
//        Log.d(TAG, "UltVerExe "+ultVerExe);
    }
    
    @Override
    protected void onStart() {
        super.onStart();
        getPreferences();
    }

    @Override
    protected void onResume() {
        super.onResume();
        if (!arrancando) terminaPausa();
//        Log.d(TAG, "onResume "+arrancando+" "+enPausa);
        arrancando = false;
    }

    @Override
    protected void onPause() {
        super.onPause();
        iniciaPausa();
//        Log.d(TAG, "onPause "+enPausa);
    }

    @Override
    protected void onStop() {
        super.onStop();
        SharedPreferences prefs =
        	PreferenceManager.getDefaultSharedPreferences(getBaseContext());
        SharedPreferences.Editor editor = prefs.edit();
        editor.putInt("AutointvItem", autointvItem);
        editor.putInt("UltVerExe", 8);
        editor.commit();
   }

    public void iniciaPausa() {
    	enPausa++;
    	if (enPausa == 1 && dadoCurrando) {
    		dadoAnimation.stop();
    		enPausaDado = true;
    	}
    }
    
    public void terminaPausa() {
    	if (enPausa == 0) return;
    	enPausa--;
    	if (enPausaDado && enPausa == 0) {
    		dadoAnimation.start();
    		enPausaDado = false;
    	}
    	if (enPausaHandle && enPausa == 0) {
    		mAutoHandler.sleep(10);
    		enPausaHandle = false;
    	}
    }
    
    public boolean restoreState(Bundle savedInstanceState) {
    	if (savedInstanceState == null) return false;
        Bundle map = savedInstanceState.getBundle(ICICLE_KEY);
        if (map == null) return false;

        blinkPawn = 0;
    	enCarrera = false;
    	enMarchaPodium = false;
    	mAutoHandler.removeMessages(0);

        autointv = map.getInt("n0aitv");
        autointvItem = map.getInt("n0aitvi");
        tipo_jugador = map.getIntArray("n0tj");
        enPausaManual = map.getBoolean("n0epm");
        mEP = new EstadoPartida();
        mTablero = new Tablero();
        mEP.restoreState(map);    	
        
        mTablero.setEstadoPartida(mEP);
        if (mEP.accion == EstadoPartida.ACC_PARADADO) {
        	mEP.accion = EstadoPartida.ACC_TIRADADO;
        }
        if (mEP.accion == EstadoPartida.ACC_MUEVE) {
            mEP.gjg = new GrupoJugadas();
            mEP.gjg.genera(mTablero, mEP.turno, mEP.vdado);
        }
        if (mEP.accion == EstadoPartida.ACC_MUEVE ||
        	mEP.accion == EstadoPartida.ACC_PASA) {
    		mTirador.getBackground().setColorFilter(colorj[mEP.turno], PorterDuff.Mode.SRC_ATOP);
    		mAuxiliar.getBackground().setColorFilter(colorj[mEP.turno], PorterDuff.Mode.SRC_ATOP);
        	mAuxiliar.invalidate();
        	pintaVdado(mEP.vdado);
        }
        mAuxiliar.setVisibility(View.GONE);
        mParchisView.setEstadoPartida(mEP);
        mParchisView.invalidate();
//        Log.d(TAG, "RestoreState "+enPausa);
        if (enPausaManual) {
            mTVOnPause.setVisibility(View.VISIBLE);
        	mBPausa.setCompoundDrawablesWithIntrinsicBounds(0, 0, R.drawable.play3, 0);
        	enPausa = 1;
        }
		mBPausa.getBackground().setColorFilter(colorj[mEP.turno], PorterDuff.Mode.SRC_ATOP);
    	mBPausa.invalidate();
        preparaSigAccion();
        return true;
    }

    public void setNewGame() {
    	blinkPawn = 0;
    	enCarrera = false;
    	enMarchaPodium = false;
    	mAutoHandler.removeMessages(0);

    	mEP = new EstadoPartida();
        mTablero = new Tablero();
        mEP.iniciaPartida(tipo_jugador[0], tipo_jugador[1],
        		          tipo_jugador[2], tipo_jugador[3]);
        mTablero.setEstadoPartida(mEP);
        mAuxiliar.setVisibility(View.GONE);
        mBPausa.setVisibility(View.GONE);
    	mBPausa.setCompoundDrawablesWithIntrinsicBounds(0, 0, R.drawable.pause3, 0);
        mTVOnPause.setVisibility(View.INVISIBLE);
        enPausa = 0;
        enPausaManual = false;
        enPausaDado = false;
        enPausaHandle = false;
        mParchisView.setEstadoPartida(mEP);
        mParchisView.invalidate();
        preparaSigAccion();
   }

    public void saveGame() {
        try {
            FileOutputStream fout = openFileOutput(SAVE_FILENAME, 0);
            ObjectOutputStream oout = new ObjectOutputStream(fout);
            Bundle bSave = new Bundle();
            
            onSaveInstanceState(bSave);
            oout.writeObject(SAVE_VERSION);
            oout.writeObject(bSave);
            oout.close();

        } catch (FileNotFoundException e) {
            ;
        } catch (IOException e) {
            ;
        }
    }
    
    public Bundle loadSavedGame() {
    	try {
            FileInputStream fin = openFileInput(SAVE_FILENAME);
            ObjectInputStream oin = new ObjectInputStream(fin);
            
            String version = (String)oin.readObject();
            if (!version.equals(SAVE_VERSION)) {
                return null;
            }
            Bundle bsave = (Bundle)oin.readObject();
            oin.close();
            return bsave;
            
        } catch (FileNotFoundException e) {
            ;
        } catch (StreamCorruptedException e) {
            ;
        } catch (IOException e) {
            ;
        } catch (ClassNotFoundException e) {
            ;
        }
        return null;
    }
    
    private AutoHandler mAutoHandler = new AutoHandler();

    class AutoHandler extends Handler {

        @Override
        public void handleMessage(Message msg) {
        	if (enPausa > 0) enPausaHandle = true;
        	else Parchis4A.this.autoUpdate();
        }

        public void sleep(long delayMillis) {
        	this.removeMessages(0);
        	sendMessageDelayed(obtainMessage(0), delayMillis);
        }
    };

    public void preparaSigAccion() {
        Resources res = getBaseContext().getResources();

    	if (mEP.tipo_jugador[mEP.turno] == EstadoPartida.PERSONA) {
	        mBPausa.setVisibility(View.GONE);
    	} else {
	        mBPausa.setVisibility(View.VISIBLE);
    	}
        if (mEP.accion == EstadoPartida.ACC_TIRADADO) {
//        	mTVEstadisticas.setText(mEP.impEstadisticas());
    		mTirador.setCompoundDrawablesWithIntrinsicBounds(R.drawable.dadox, 0, 0, 0);
    		mTirador.getBackground().setColorFilter(colorj[mEP.turno], PorterDuff.Mode.SRC_ATOP);
    		mAuxiliar.getBackground().setColorFilter(colorj[mEP.turno], PorterDuff.Mode.SRC_ATOP);
        	mAuxiliar.invalidate();
    		mBPausa.getBackground().setColorFilter(colorj[mEP.turno], PorterDuff.Mode.SRC_ATOP);
        	mBPausa.invalidate();
        	if (mEP.tipo_jugador[mEP.turno] == EstadoPartida.PERSONA) {
    		    mTirador.setText(res.getString(R.string.acc_tira));
        	} else {
    		    mTirador.setText(res.getString(R.string.acc_tiro));
        		mAutoHandler.sleep(100*autointv);
        	}
            mParchisView.invalidate();
    	} else if (mEP.accion == EstadoPartida.ACC_PARADADO) {
    		mTirador.setText(res.getString(R.string.acc_espera));
    		mAutoHandler.sleep(100*autointv);
    	} else if (mEP.accion == EstadoPartida.ACC_MUEVE) {
        	if (mEP.tipo_jugador[mEP.turno] == EstadoPartida.PERSONA) {
            	mParchisView.mustCalcPawnJgPos = true;
            	mParchisView.showPawnCruz = true;
                if (mEP.gjg.jg[0].acasa)
                	mTirador.setText(res.getString(R.string.acc_acasa));
                else if (mEP.gjg.jg[0].entra) {
                	if (mEP.ha_salido[mEP.gjg.jg[0].cjugador])
                	    mTirador.setText(res.getString(R.string.acc_sales));
                	else
            	        mTirador.setText(res.getString(R.string.acc_sales2));
                } else if (mEP.gjg.njugadas > 1) {
                    mAuxiliar.setVisibility(View.VISIBLE);
        		    mTirador.setText(res.getString(R.string.acc_mueve2));
                } else
        		    mTirador.setText(res.getString(R.string.acc_mueve));
                if (mEP.gjg.njugadas > 0) {
        		    enCarrera = true;
        		    mCarrera = null;
                }
        	} else {
                if (mEP.gjg.jg[0].acasa)
                	mTirador.setText(res.getString(R.string.acc_acasam));
                else if (mEP.gjg.jg[0].entra) {
                	if (mEP.ha_salido[mEP.gjg.jg[0].cjugador])
                	    mTirador.setText(res.getString(R.string.acc_salgo));
                	else
                	    mTirador.setText(res.getString(R.string.acc_salgo2));
                } else
        		    mTirador.setText(res.getString(R.string.acc_muevo));
                if (mEP.gjg.njugadas > 0) {
                	blinkPawn = 1;
                	mParchisView.showPawnCruz = true;
        		    mAutoHandler.sleep(18*autointv);
        		    enCarrera = true;
        		    mCarrera = null;
                } else
        		    mAutoHandler.sleep(250*autointv);
        	}
            mParchisView.invalidate();
    	} else if (mEP.accion == EstadoPartida.ACC_PASA) {
        	if (mEP.tipo_jugador[mEP.turno] == EstadoPartida.PERSONA) {
        		mTirador.setText(res.getString(R.string.acc_pasas));
        	} else {
        		mTirador.setText(res.getString(R.string.acc_paso));
        		mAutoHandler.sleep(200*autointv);
        	}
    	} else {
	        mBPausa.setVisibility(View.GONE);
    		mTirador.setCompoundDrawablesWithIntrinsicBounds(R.drawable.dadox, 0, 0, 0);
    		mTirador.getBackground().setColorFilter(0xff777777, PorterDuff.Mode.SRC_ATOP);
    		mTirador.setText(res.getString(R.string.acc_fin));
//    		mAuxiliar.getBackground().setColorFilter(0xff777777, PorterDuff.Mode.SRC_ATOP);
//        	mAuxiliar.invalidate();
    		if (mParchisView.showMunecos) {
    			mAutoHandler.sleep(12*autointv);
    			enMarchaPodium = true;
        	    mMarchaPodium = null;
    		} else {
        	    Animation mAnimation = AnimationUtils.loadAnimation(this, R.anim.salto);
        	    mParchisView.startAnimation(mAnimation);
    		}
    	}
    }

    private void pintaVdado(int vdado) {
		int res;
		if (vdado == 1) res = R.drawable.dado1;
		else if (vdado == 2) res = R.drawable.dado2;
		else if (vdado == 3) res = R.drawable.dado3;
		else if (vdado == 4) res = R.drawable.dado4;
		else if (vdado == 5) res = R.drawable.dado5;
		else if (vdado == 6) res = R.drawable.dado6;
		else if (vdado == 7) res = R.drawable.dado6;
		else if (vdado == 10) res = R.drawable.dado10;
		else if (vdado == 20) res = R.drawable.dado20;
		else return;
		mTirador.setCompoundDrawablesWithIntrinsicBounds(res, 0, 0, 0);
    }
    
    public void ajustaAccion() {
        if (mEP.accion == EstadoPartida.ACC_TIRADADO) {
    	    mTirador.setCompoundDrawablesWithIntrinsicBounds(R.drawable.mueve_dado, 0, 0, 0);
        	Drawable bdraw[] = mTirador.getCompoundDrawables();
        	dadoAnimation = (AnimationDrawable) bdraw[0];
    	    dadoAnimation.start();
    	    dadoCurrando = true;
        }
        if (mEP.accion == EstadoPartida.ACC_PARADADO) {
        	Drawable bdraw[] = mTirador.getCompoundDrawables();
        	dadoAnimation = (AnimationDrawable) bdraw[0];
    	    dadoAnimation.stop();
    	    pintaVdado(mEP.vdado);
    	    dadoCurrando = false;
        }
        if (mEP.accion == EstadoPartida.ACC_MUEVE) {
        	if (mEP.sigvdado >= 10) {
        		pintaVdado(mEP.sigvdado);
        	}
            mAuxiliar.setVisibility(View.GONE);
            mParchisView.invalidate();
        }
    }
    
    public boolean doBlinkPawn() {
    	if (blinkPawn == 0) return false;
      	mParchisView.showPawnCruz = !mParchisView.showPawnCruz;
        mParchisView.invalidate();
        if (blinkPawn > 7)
        	blinkPawn = 0;
        else
        	blinkPawn++;
        mAutoHandler.sleep(18*autointv);
        return true;
    }

    public boolean doEnCarrera() {
    	int intv = 12;
    	
    	if (!enCarrera) return false;
    	if (mCarrera == null)
    		mCarrera = new Carrera(mEP.gjg.jg[mEP.njgelegida]);
    	if (mCarrera == null || mCarrera.nula) {
    		enCarrera = false;
    		return false;
    	}
    	if (mCarrera.sigpaso >= mCarrera.npasos) {
    		if (!mCarrera.especial || mCarrera.cuentaespecial > 5) {
    		    enCarrera = false;
        	    mParchisView.modoCarrera = false;
    		    return false;
    		}
    		intv = (mCarrera.cuentaespecial % 2 == 0) ? 24 : 10;
    	}
    	mParchisView.modoCarrera = true;
    	mParchisView.setCarrera(mCarrera);
    	mParchisView.invalidate();
        mAutoHandler.sleep(intv*autointv);
        return true;
    }

    public boolean doEnMarchaPodium() {
    	if (!enMarchaPodium) return false;
    	if (mMarchaPodium == null) {
    		mMarchaPodium = new MarchaPodium(mEP);
    	    if (mMarchaPodium == null || mMarchaPodium.njg == 0) {
    		    enMarchaPodium = false;
    		    mMarchaPodium = null;
    		    return true;
    	    }
    	} else {
    		mMarchaPodium.avanza();
    	}
    	if (mMarchaPodium.fin) {
    		enMarchaPodium = false;
    		mParchisView.modoMarchaPodium = false;
        	mParchisView.invalidate();
    	    Animation mAnimation = AnimationUtils.loadAnimation(this, R.anim.zoompod);
    	    mParchisView.startAnimation(mAnimation);
    		return true;
    	}
    	mParchisView.modoMarchaPodium = true;
    	mParchisView.setMarchaPodium(mMarchaPodium);
    	mParchisView.invalidate();
        mAutoHandler.sleep(12*autointv);
        return true;
    }
    
    public void autoUpdate() {
    	if (doBlinkPawn()) return;
    	if (doEnCarrera()) return;
    	if (doEnMarchaPodium()) return;
    	mEP.iniciaAccion(mTablero);
    	ajustaAccion();
    	mEP.terminaAccion(mTablero);
    	preparaSigAccion();
    }
    
    public void bTirador(View view) {
        Resources res = getBaseContext().getResources();

        if (enPausa > 0) {
    		Toast.makeText(this, R.string.toast_onpause, Toast.LENGTH_SHORT).show();
        	return;
        }
        if (mEP.accion == EstadoPartida.ACC_PARTIDATERMINADA) {
    		CharSequence str = res.getString(R.string.toast_gameover);
    		Toast t = Toast.makeText(this, str, Toast.LENGTH_SHORT);
    		t.setGravity(Gravity.CENTER, 0, 0);
    		t.show();;
            if (!enMarchaPodium) {
          	    Animation mAnimation = AnimationUtils.loadAnimation(this, R.anim.vuelta);
        	    mParchisView.startAnimation(mAnimation);
        		if (mParchisView.showMunecos) {
        			mAutoHandler.sleep(12*autointv);
        			enMarchaPodium = true;
            	    mMarchaPodium = null;
        		}
            }
//        	mTirador.setVisibility(View.GONE);
//        	for (int i =0; i<6; i++) mBDadoN[i].setVisibility(View.VISIBLE);
    		return;
    	}
        if (mEP.tipo_jugador[mEP.turno] != EstadoPartida.PERSONA) {
        	boolean anyperson = false;
    		for (int c=EstadoPartida.AMAR; c<=EstadoPartida.VERD; c++) {
    			if (mEP.tipo_jugador[c] == EstadoPartida.PERSONA)
    				anyperson = true;
    		}
    		CharSequence str;
    		if (anyperson)
    			str = res.getString(R.string.toast_waitturn);
    		else
    			str = res.getString(R.string.toast_stopauto);
    		Toast.makeText(this, str, Toast.LENGTH_SHORT).show();
    		return;
        }
    	if (mEP.accion == EstadoPartida.ACC_PARADADO) return;

        if (doEnCarrera()) return;
    	mEP.iniciaAccion(mTablero);
    	ajustaAccion();
    	mEP.terminaAccion(mTablero);
    	preparaSigAccion();
    }

    public void bAuxiliar(View view) {
        if (enPausa > 0) return;
        mEP.njgelegida++;
        if (mEP.njgelegida >= mEP.gjg.njugadas) mEP.njgelegida = 0;
        mParchisView.invalidate();
    }

    public void bPausa(View view) {
    	if (mBPausa.getVisibility() != View.VISIBLE)
    		return;
        if (enPausa > 0) {
        	terminaPausa();
        	enPausaManual = false;
        	mBPausa.setCompoundDrawablesWithIntrinsicBounds(0, 0, R.drawable.pause3, 0);
            mTVOnPause.setVisibility(View.INVISIBLE);
        	}
        else {
        	iniciaPausa();
        	enPausaManual = true;
        	mBPausa.setCompoundDrawablesWithIntrinsicBounds(0, 0, R.drawable.play3, 0);
            mTVOnPause.setVisibility(View.VISIBLE);
        }
    }
/*
    public void bDado1(View view) {
        mParchisView.invalidate();
    }

    public void bDado2(View view) {
        mParchisView.invalidate();
    }

    public void bDado3(View view) {
        mParchisView.invalidate();
    }

    public void bDado4(View view) {
        mParchisView.invalidate();
    }

    public void bDado5(View view) {
        mParchisView.invalidate();
    }

    public void bDado6(View view) {
        mParchisView.invalidate();
    }
*/
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.parchis4a_menu, menu);
    	menu.findItem(R.id.menu_about).setIcon(
        	getResources().getDrawable(android.R.drawable.ic_menu_info_details));
    	menu.findItem(R.id.menu_settings).setIcon(
            getResources().getDrawable(android.R.drawable.ic_menu_preferences));
    	menu.findItem(R.id.menu_help).setIcon(
            getResources().getDrawable(android.R.drawable.ic_menu_help));
    	menu.findItem(R.id.menu_selectplayers).setIcon(
            getResources().getDrawable(android.R.drawable.ic_menu_set_as));
    	menu.findItem(R.id.menu_stats).setIcon(
                getResources().getDrawable(android.R.drawable.ic_menu_search));
    	menu.findItem(R.id.menu_fast).setIcon(
                getResources().getDrawable(android.R.drawable.ic_menu_view));
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle item selection
        switch (item.getItemId()) {
        case R.id.menu_selectplayers:
        	showDialog(DIALOG_SELECTPLAYERS_ID);
            return true;
        case R.id.menu_stats:
        	showDialog(DIALOG_STATS_ID);
       	    return true;
        case R.id.menu_fast:
        	showDialog(DIALOG_GAMESPEED_ID);
       	    return true;
        case R.id.menu_settings:
        	  Intent settingsActivity = new Intent(getBaseContext(), Preferences.class);
              startActivity(settingsActivity);
              return true;
        case R.id.menu_about:
        	showDialog(DIALOG_ABOUT_ID);
            return true;
        case R.id.menu_help:
        	showDialog(DIALOG_HELP_ID);
            return true;
       default:
            return super.onOptionsItemSelected(item);
        }
    }
    
    @Override
    protected Dialog onCreateDialog(int id) {
        Dialog dialog;

        switch(id) {
        case DIALOG_SELECTPLAYERS_ID:
        	dialog = new Dialog(this);
        	dialog.setContentView(R.layout.select_players);
        	dialog.setCancelable(true);
            dialog.setCanceledOnTouchOutside(true);
        	dialog.setTitle(resG.getString(R.string.select_players_title));
        	mRG_tipo_jugador[0] = (RadioGroup) dialog.findViewById(R.id.radioGroup1);
        	mRG_tipo_jugador[1] = (RadioGroup) dialog.findViewById(R.id.radioGroup2);
        	mRG_tipo_jugador[2] = (RadioGroup) dialog.findViewById(R.id.radioGroup3);
        	mRG_tipo_jugador[3] = (RadioGroup) dialog.findViewById(R.id.radioGroup4);
        	for (int j=0; j<4; j++) {
        		if (tipo_jugador[j] == EstadoPartida.PERSONA) mRG_tipo_jugador[j].check(R.id.radio0);
        		else if (tipo_jugador[j] == EstadoPartida.MAQUINA) mRG_tipo_jugador[j].check(R.id.radio1);
        		else mRG_tipo_jugador[j].check(R.id.radio2);
        	}

            Button selectpDialogOkButton = (Button) dialog.findViewById(R.id.select_players_ok_button);
            selectpDialogOkButton.setOnClickListener(new Button.OnClickListener() {
                public void onClick(View v) {
                	for (int j=0; j<4; j++) {
                    	int id = mRG_tipo_jugador[j].getCheckedRadioButtonId();
                		if (id == R.id.radio0) tipo_jugador[j] = EstadoPartida.PERSONA;
                		else if (id == R.id.radio1) tipo_jugador[j] = EstadoPartida.MAQUINA;
                		else tipo_jugador[j] = EstadoPartida.NOJUEGA;
                	}
                    setNewGame();
                    dismissDialog(DIALOG_SELECTPLAYERS_ID);
                }
            });

            Button selectpDialogCancelButton = (Button) dialog.findViewById(R.id.select_players_cancel_button);
            selectpDialogCancelButton.setOnClickListener(new Button.OnClickListener() {
                public void onClick(View v) {
                    dismissDialog(DIALOG_SELECTPLAYERS_ID);
                }
            });

            break;
        case DIALOG_STATS_ID:
        	dialog = new Dialog(this);
        	dialog.setContentView(R.layout.stats_layout);
        	dialog.setCancelable(true);
            dialog.setCanceledOnTouchOutside(true);
        	dialog.setTitle(resG.getString(R.string.stats_title));

        	dialog.setOnShowListener(new DialogInterface.OnShowListener() {
        		public void onShow(DialogInterface dialog) {
                	iniciaPausa();
        		}
        	});
        	dialog.setOnDismissListener(new DialogInterface.OnDismissListener() {
        		public void onDismiss(DialogInterface dialog) {
        			terminaPausa();
        		}
        	});
            Button statsDialogOkButton = (Button) dialog.findViewById(R.id.stats_ok_button);
            statsDialogOkButton.setOnClickListener(new Button.OnClickListener() {
                public void onClick(View v) {
                    dismissDialog(DIALOG_STATS_ID);
                }
            });
            break;
        case DIALOG_ABOUT_ID:
        	dialog = new Dialog(this);
        	dialog.setContentView(R.layout.about_layout);
        	dialog.setCancelable(true);
            dialog.setCanceledOnTouchOutside(true);
        	dialog.setTitle(resG.getString(R.string.about_title));

        	dialog.setOnShowListener(new DialogInterface.OnShowListener() {
        		public void onShow(DialogInterface dialog) {
                	iniciaPausa();
        		}
        	});
        	dialog.setOnDismissListener(new DialogInterface.OnDismissListener() {
        		public void onDismiss(DialogInterface dialog) {
        			terminaPausa();
        		}
        	});
            Button aboutDialogOkButton = (Button) dialog.findViewById(R.id.about_ok_button);
            aboutDialogOkButton.setOnClickListener(new Button.OnClickListener() {
                public void onClick(View v) {
                    dismissDialog(DIALOG_ABOUT_ID);
                }
            });
            break;
        case DIALOG_NEW_ID:
        	dialog = new Dialog(this);
        	dialog.setContentView(R.layout.new_layout);
        	dialog.setCancelable(true);
            dialog.setCanceledOnTouchOutside(true);
        	dialog.setTitle(resG.getString(R.string.new_title));

/*        	dialog.setOnDismissListener(new DialogInterface.OnDismissListener() {
        		public void onDismiss(DialogInterface dialog) {
                    showDialog(DIALOG_WELCOME_ID);
        		}
        	});*/
            Button newDialogOkButton = (Button) dialog.findViewById(R.id.new_ok_button);
            newDialogOkButton.setOnClickListener(new Button.OnClickListener() {
                public void onClick(View v) {
                    dismissDialog(DIALOG_NEW_ID);
                }
            });
            break;
        case DIALOG_HELP_ID:
        	dialog = new Dialog(this);
        	dialog.setContentView(R.layout.help_layout);
        	dialog.setCancelable(true);
            dialog.setCanceledOnTouchOutside(true);
        	dialog.setTitle(resG.getString(R.string.help_title));

        	dialog.setOnShowListener(new DialogInterface.OnShowListener() {
        		public void onShow(DialogInterface dialog) {
                	iniciaPausa();
        		}
        	});
        	dialog.setOnDismissListener(new DialogInterface.OnDismissListener() {
        		public void onDismiss(DialogInterface dialog) {
        			terminaPausa();
        		}
        	});
            Button helpDialogOkButton = (Button) dialog.findViewById(R.id.help_ok_button);
            helpDialogOkButton.setOnClickListener(new Button.OnClickListener() {
                public void onClick(View v) {
                    dismissDialog(DIALOG_HELP_ID);
                }
            });
            break;
        case DIALOG_EXIT_ID:
        	AlertDialog.Builder builderde = new AlertDialog.Builder(this);
        	builderde.setMessage(resG.getString(R.string.dlg_exit_title));
        	builderde.setCancelable(false);
        	builderde.setPositiveButton(resG.getString(R.string.dlg_yes),
        	    new DialogInterface.OnClickListener() {
        	        public void onClick(DialogInterface dialog, int id) {
//        	            saveGame();
        	            finish();
        	        }
        	    });
        	builderde.setNegativeButton(resG.getString(R.string.dlg_no),
        	    new DialogInterface.OnClickListener() {
        	        public void onClick(DialogInterface dialog, int id) {
    	        	    terminaPausa();
   	                    dialog.cancel();
        	        }
        	    });
// 	               .setNeutralButton(res.getString(R.string.dlg_cancel),
//    	    	       new DialogInterface.OnClickListener() {
//    	               public void onClick(DialogInterface dialog, int id) {
//    	        	       terminaPausa();
//   	                   dialog.cancel();
//    	               }
//    	           });
        	dialog = builderde.create();
        	dialog.setOnShowListener(new DialogInterface.OnShowListener() {
        		public void onShow(DialogInterface dialog) {
                	iniciaPausa();
        		}
        	});
        	break;
        case DIALOG_GAMESPEED_ID:
        	AlertDialog.Builder buildergs = new AlertDialog.Builder(this);
        	buildergs.setTitle(resG.getString(R.string.dlg_gamespeed_title));
        	buildergs.setSingleChoiceItems(R.array.dlg_gamespeed_item, autointvItem,
        		new DialogInterface.OnClickListener() {
        	    public void onClick(DialogInterface dialog, int item) {
        	    	autointvItem = item;
        	    	autointv = resG.getIntArray(R.array.dlg_gamespeed_value)[item];
        	    	String s1 = resG.getString(R.string.toast_gamespeed);
        	    	String s2 = resG.getStringArray(R.array.dlg_gamespeed_item)[item];
        	        Toast.makeText(getApplicationContext(), s1+" "+s2, Toast.LENGTH_SHORT).show();
        	    	dialog.cancel();
        	    }
        	});
 	        buildergs.setPositiveButton(resG.getString(R.string.dlg_cancel),
    	    	new DialogInterface.OnClickListener() {
    	        public void onClick(DialogInterface dialog, int id) {
        	    	dialog.cancel();
    	        }
    	    });
        	dialog = buildergs.create();
        	dialog.setOnShowListener(new DialogInterface.OnShowListener() {
        		public void onShow(DialogInterface dialog) {
                	iniciaPausa();
        		}
        	});
        	dialog.setOnDismissListener(new DialogInterface.OnDismissListener() {
        		public void onDismiss(DialogInterface dialog) {
        			terminaPausa();
        		}
        	});
        	break;
        case DIALOG_WELCOME_ID:
        	dialog = new Dialog(this);
        	dialog.setContentView(R.layout.welcome_layout);
        	dialog.setCancelable(true);
        	dialog.setTitle(resG.getString(R.string.welcome_title));

            Button welcomeBeginButton = (Button) dialog.findViewById(R.id.welcome_begin_button);
            welcomeBeginButton.setOnClickListener(new Button.OnClickListener() {
                public void onClick(View v) {
                    dismissDialog(DIALOG_WELCOME_ID);
                }
            });
            Button welcomeOneButton = (Button) dialog.findViewById(R.id.welcome_one_button);
            welcomeOneButton.setOnClickListener(new Button.OnClickListener() {
                public void onClick(View v) {
            		tipo_jugador[0] = EstadoPartida.NOJUEGA;
            		tipo_jugador[1] = EstadoPartida.PERSONA;
            		tipo_jugador[2] = EstadoPartida.NOJUEGA;
            		tipo_jugador[3] = EstadoPartida.MAQUINA;
                    setNewGame();
                    dismissDialog(DIALOG_WELCOME_ID);
                }
            });
            Button welcomeSelectButton = (Button) dialog.findViewById(R.id.welcome_select_button);
            welcomeSelectButton.setOnClickListener(new Button.OnClickListener() {
                public void onClick(View v) {
              	    showDialog(DIALOG_SELECTPLAYERS_ID);
                    dismissDialog(DIALOG_WELCOME_ID);
                }
            });
            Button welcomeHelpButton = (Button) dialog.findViewById(R.id.welcome_help_button);
            welcomeHelpButton.setOnClickListener(new Button.OnClickListener() {
                public void onClick(View v) {
                    showDialog(DIALOG_HELP_ID);
                    dismissDialog(DIALOG_WELCOME_ID);
                }
            });
            Button newHelpButton = (Button) dialog.findViewById(R.id.welcome_new_button);
            if (ultVerExe != 8) {
            	newHelpButton.setTextColor(0xffff0000);
            }
            newHelpButton.setOnClickListener(new Button.OnClickListener() {
                public void onClick(View v) {
                    showDialog(DIALOG_NEW_ID);
//                    dismissDialog(DIALOG_WELCOME_ID);
                }
            });
            break;
        default:
            dialog = null;
        }
        return dialog;
    }

    int ridEstDado[][] = {
    	{R.id.tv0d1, R.id.tv0d2, R.id.tv0d3, R.id.tv0d4, R.id.tv0d5, R.id.tv0d6},
    	{R.id.tv1d1, R.id.tv1d2, R.id.tv1d3, R.id.tv1d4, R.id.tv1d5, R.id.tv1d6},
    	{R.id.tv2d1, R.id.tv2d2, R.id.tv2d3, R.id.tv2d4, R.id.tv2d5, R.id.tv2d6},
    	{R.id.tv3d1, R.id.tv3d2, R.id.tv3d3, R.id.tv3d4, R.id.tv3d5, R.id.tv3d6} };
    int ridEstPasa[]= {R.id.tv0lj1, R.id.tv1lj1, R.id.tv2lj1, R.id.tv3lj1}; 
    int ridEstCome[]= {R.id.tv0lj2, R.id.tv1lj2, R.id.tv2lj2, R.id.tv3lj2}; 
    int ridEstComida[]= {R.id.tv0lj3, R.id.tv1lj3, R.id.tv2lj3, R.id.tv3lj3}; 
    int ridEstACasa[]= {R.id.tv0lj4, R.id.tv1lj4, R.id.tv2lj4, R.id.tv3lj4}; 
    int ridEstPOS[]= {R.id.tv0lj5, R.id.tv1lj5, R.id.tv2lj5, R.id.tv3lj5}; 

    @Override
    protected void onPrepareDialog(int id, Dialog dialog) {
        switch(id) {
        case DIALOG_STATS_ID:
        	TextView mTView;
        	String s;
    		for (int c=EstadoPartida.AMAR; c<=EstadoPartida.VERD; c++) {
    			boolean mostrar = mEP.tipo_jugador[c] != EstadoPartida.NOJUEGA;
    			for (int ind=0; ind<6; ind++) {
                    mTView = (TextView) dialog.findViewById(ridEstDado[c][ind]);
                    s = mostrar ?
                    	String.valueOf( (int)(((mEP.estDado[c][ind] * 100.0) / mEP.estTotDado[c]) + 0.5) ) + "%"
                    	: "-";
                    mTView.setText(s);
    			}
                mTView = (TextView) dialog.findViewById(ridEstPasa[c]);
                mTView.setText(mostrar ? String.valueOf(mEP.estPasa[c]) : "-");
                mTView = (TextView) dialog.findViewById(ridEstCome[c]);
                mTView.setText(mostrar ? String.valueOf(mEP.estCome[c]) : "-");
                mTView = (TextView) dialog.findViewById(ridEstComida[c]);
                mTView.setText(mostrar ? String.valueOf(mEP.estComida[c]) : "-");
                mTView = (TextView) dialog.findViewById(ridEstACasa[c]);
                mTView.setText(mostrar ? String.valueOf(mEP.estACasa[c]) : "-");
                mTView = (TextView) dialog.findViewById(ridEstPOS[c]);
                s = (!mostrar || mEP.ha_terminado[c] < 0) ? "-" : (mEP.ha_terminado[c] + 1) + "º";
                mTView.setText(s);
    		}
            break;
        }
    }

    @Override
    public void onBackPressed() {
    	if (mEP.haComenzado() && !mEP.haTerminado())
    	    showDialog(DIALOG_EXIT_ID);
    	else
    	    finish();
    }
}